﻿using System.Drawing;

namespace TestProject
{
    /// <summary>
    /// 스테가노그래피 헬퍼
    /// </summary>
    public static class SteganographyHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 비트맵 구하기 - GetBitmap(sourceText, sourceBitmap)

        /// <summary>
        /// 비트맵 구하기
        /// </summary>
        /// <param name="sourceText">소스 텍스트</param>
        /// <param name="sourceBitmap">소스 비트맵</param>
        /// <returns>비트맵</returns>
        public static Bitmap GetBitmap(string sourceText, Bitmap sourceBitmap)
        {
            SteganographyState state = SteganographyState.HIDING;

            int  characterIndex = 0;
            int  characterValue = 0;
            long colorUnitIndex = 0;
            int  zeros          = 0;
            int  red            = 0;
            int  green          = 0;
            int  blue           = 0;

            Bitmap targetBitmap = sourceBitmap.Clone() as Bitmap;

            for(int y = 0; y < sourceBitmap.Height; y++)
            {
                for(int x = 0; x < sourceBitmap.Width; x++)
                {
                    Color pixel = sourceBitmap.GetPixel(x, y);

                    pixel = Color.FromArgb(pixel.R - pixel.R % 2, pixel.G - pixel.G % 2, pixel.B - pixel.B % 2);

                    red   = pixel.R;
                    green = pixel.G;
                    blue  = pixel.B;

                    for(int i = 0; i < 3; i++)
                    {
                        if(colorUnitIndex % 8 == 0)
                        {
                            if(zeros == 8)
                            {
                                if((colorUnitIndex - 1) % 3 < 2)
                                {
                                    targetBitmap.SetPixel(x, y, Color.FromArgb(red, green, blue));
                                }

                                return targetBitmap;
                            }

                            if(characterIndex >= sourceText.Length)
                            {
                                state = SteganographyState.FILLING_WITH_ZEROS;
                            }
                            else
                            {
                                characterValue = sourceText[characterIndex++];
                            }
                        }

                        switch(colorUnitIndex % 3)
                        {
                            case 0 :
                            {
                                if(state == SteganographyState.HIDING)
                                {
                                    red += characterValue % 2;

                                    characterValue /= 2;
                                }

                                break;
                            }
                            case 1 :
                            {
                                if(state == SteganographyState.HIDING)
                                {
                                    green += characterValue % 2;

                                    characterValue /= 2;
                                }

                                break;
                            }
                            case 2 :
                            {
                                if(state == SteganographyState.HIDING)
                                {
                                    blue += characterValue % 2;

                                    characterValue /= 2;
                                }

                                targetBitmap.SetPixel(x, y, Color.FromArgb(red, green, blue));

                                break;
                            }
                        }

                        colorUnitIndex++;

                        if(state == SteganographyState.FILLING_WITH_ZEROS)
                        {
                            zeros++;
                        }
                    }
                }
            }

            return targetBitmap;
        }

        #endregion
        #region 텍스트 구하기 - GetText(sourceBitmap)

        /// <summary>
        /// 텍스트 구하기
        /// </summary>
        /// <param name="sourceBitmap">소스 비트맵</param>
        /// <returns>텍스트</returns>
        public static string GetText(Bitmap sourceBitmap)
        {
            int    colorUnitIndex = 0;
            int    characterValue = 0;
            string targetText     = string.Empty;

            for(int y = 0; y < sourceBitmap.Height; y++)
            {
                for(int x = 0; x < sourceBitmap.Width; x++)
                {
                    Color pixel = sourceBitmap.GetPixel(x, y);

                    for(int i = 0; i < 3; i++)
                    {
                        switch(colorUnitIndex % 3)
                        {
                            case 0 :
                            {
                                characterValue = characterValue * 2 + pixel.R % 2;

                                break;
                            }
                            case 1 :
                            {
                                characterValue = characterValue * 2 + pixel.G % 2;

                                break;
                            }
                            case 2 :
                            {
                                characterValue = characterValue * 2 + pixel.B % 2;

                                break;
                            }
                        }

                        colorUnitIndex++;

                        if(colorUnitIndex % 8 == 0)
                        {
                            characterValue = ReverseBits(characterValue);

                            if(characterValue == 0)
                            {
                                return targetText;
                            }

                            char character = (char)characterValue;

                            targetText += character.ToString();
                        }
                    }
                }
            }

            return targetText;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 비트 반전하기 - ReverseBits(sourceValue)

        /// <summary>
        /// 비트 반전하기
        /// </summary>
        /// <param name="sourceValue">소스 값</param>
        /// <returns>비트 반전 값</returns>
        public static int ReverseBits(int sourceValue)
        {
            int targetValue = 0;

            for(int i = 0; i < 8; i++)
            {
                targetValue = targetValue * 2 + sourceValue % 2;

                sourceValue /= 2;
            }

            return targetValue;
        }

        #endregion
    }
}